@@ -1,7 +1,10 @@
 from typing import Any, Dict, Literal, Mapping
 
 
-Danmaku = Mapping[str, Any]
+ApiPlatform = Literal[
+    'web',
+    'android',
+]
 
 QualityNumber = Literal[
     20000,  # 4K
@@ -15,8 +18,16 @@ QualityNumber = Literal[
 
 StreamFormat = Literal[
     'flv',
+    'ts',
     'fmp4',
 ]
 
+StreamCodec = Literal[
+    'avc',
+    'hevc',
+]
+
 JsonResponse = Dict[str, Any]
 ResponseData = Dict[str, Any]
+
+Danmaku = Mapping[str, Any]
