@@ -1,27 +1,32 @@
-import html
 import asyncio
+import html
 import logging
 from contextlib import suppress
+from threading import Lock
 from typing import Iterator, List, Optional
 
+from tenacity import AsyncRetrying, retry_if_not_exception_type, stop_after_attempt
+
+from blrec import __github__, __prog__, __version__
+from blrec.bili.live import Live
 from blrec.core.models import GiftSendMsg, GuardBuyMsg, SuperChatMsg
-
-
-from .. import __version__, __prog__, __github__
-from .danmaku_receiver import DanmakuReceiver, DanmuMsg
-from .stream_recorder import StreamRecorder, StreamRecorderEventListener
-from .statistics import StatisticsCalculator
-from ..bili.live import Live
-from ..exception import exception_callback
-from ..event.event_emitter import EventListener, EventEmitter
-from ..path import danmaku_path
-from ..danmaku.models import (
-    Metadata, Danmu, GiftSendRecord, GuardBuyRecord, SuperChatRecord
+from blrec.danmaku.io import DanmakuWriter
+from blrec.danmaku.models import (
+    Danmu,
+    GiftSendRecord,
+    GuardBuyRecord,
+    Metadata,
+    SuperChatRecord,
 )
-from ..danmaku.io import DanmakuWriter
-from ..utils.mixins import SwitchableMixin
-from ..logging.room_id import aio_task_with_room_id
+from blrec.event.event_emitter import EventEmitter, EventListener
+from blrec.exception import exception_callback, submit_exception
+from blrec.logging.room_id import aio_task_with_room_id
+from blrec.path import danmaku_path
+from blrec.utils.mixins import SwitchableMixin
 
+from .danmaku_receiver import DanmakuReceiver, DanmuMsg
+from .statistics import Statistics
+from .stream_recorder import StreamRecorder, StreamRecorderEventListener
 
 __all__ = 'DanmakuDumper', 'DanmakuDumperEventListener'
 
@@ -50,6 +55,7 @@ class DanmakuDumper(
         *,
         danmu_uname: bool = False,
         record_gift_send: bool = False,
+        record_free_gifts: bool = False,
         record_guard_buy: bool = False,
         record_super_chat: bool = False,
     ) -> None:
@@ -61,24 +67,26 @@ class DanmakuDumper(
 
         self.danmu_uname = danmu_uname
         self.record_gift_send = record_gift_send
+        self.record_free_gifts = record_free_gifts
         self.record_guard_buy = record_guard_buy
         self.record_super_chat = record_super_chat
 
+        self._lock: Lock = Lock()
         self._path: Optional[str] = None
         self._files: List[str] = []
-        self._calculator = StatisticsCalculator(interval=60)
+        self._statistics = Statistics(interval=60)
 
     @property
-    def danmu_count(self) -> int:
-        return self._calculator.count
+    def danmu_total(self) -> int:
+        return self._statistics.count
 
     @property
     def danmu_rate(self) -> float:
-        return self._calculator.rate
+        return self._statistics.rate
 
     @property
     def elapsed(self) -> float:
-        return self._calculator.elapsed
+        return self._statistics.elapsed
 
     @property
     def dumping_path(self) -> Optional[str]:
@@ -86,10 +94,13 @@ class DanmakuDumper(
 
     def _do_enable(self) -> None:
         self._stream_recorder.add_listener(self)
+        self._statistics.reset()
         logger.debug('Enabled danmaku dumper')
 
     def _do_disable(self) -> None:
         self._stream_recorder.remove_listener(self)
+        asyncio.create_task(self._stop_dumping())
+        self._statistics.freeze()
         logger.debug('Disabled danmaku dumper')
 
     def set_live_start_time(self, time: int) -> None:
@@ -108,23 +119,45 @@ class DanmakuDumper(
     async def on_video_file_created(
         self, video_path: str, record_start_time: int
     ) -> None:
-        self._path = danmaku_path(video_path)
-        self._record_start_time = record_start_time
-        self._files.append(self._path)
-        self._start_dumping()
+        with self._lock:
+            self._delta: float = 0
+            self._record_start_time: int = record_start_time
+            self._timebase: int = self._record_start_time * 1000
+            self._stream_recording_interrupted: bool = False
+            self._path = danmaku_path(video_path)
+            self._files.append(self._path)
+            await self._stop_dumping()
+            self._start_dumping()
 
     async def on_video_file_completed(self, video_path: str) -> None:
-        await self._stop_dumping()
-        self._path = None
+        with self._lock:
+            await self._stop_dumping()
+            self._path = None
+
+    async def on_stream_recording_interrupted(self, duration: float) -> None:
+        logger.debug(f'Stream recording interrupted, {duration}')
+        self._duration = duration
+        self._stream_recording_recovered = asyncio.Condition()
+        self._stream_recording_interrupted = True
+
+    async def on_stream_recording_recovered(self, timestamp: int) -> None:
+        logger.debug(f'Stream recording recovered, {timestamp}')
+        self._timebase = timestamp * 1000
+        self._delta = self._duration * 1000
+        self._stream_recording_interrupted = False
+        async with self._stream_recording_recovered:
+            self._stream_recording_recovered.notify_all()
 
     def _start_dumping(self) -> None:
         self._create_dump_task()
 
     async def _stop_dumping(self) -> None:
-        await self._cancel_dump_task()
+        if hasattr(self, '_dump_task'):
+            await self._cancel_dump_task()
+            del self._dump_task  # type: ignore
 
     def _create_dump_task(self) -> None:
-        self._dump_task = asyncio.create_task(self._dump())
+        self._dump_task = asyncio.create_task(self._do_dump())
         self._dump_task.add_done_callback(exception_callback)
 
     async def _cancel_dump_task(self) -> None:
@@ -133,10 +166,9 @@ class DanmakuDumper(
             await self._dump_task
 
     @aio_task_with_room_id
-    async def _dump(self) -> None:
+    async def _do_dump(self) -> None:
         assert self._path is not None
         logger.debug('Started dumping danmaku')
-        self._calculator.reset()
 
         try:
             async with DanmakuWriter(self._path) as writer:
@@ -144,36 +176,52 @@ class DanmakuDumper(
                 await self._emit('danmaku_file_created', self._path)
                 await writer.write_metadata(self._make_metadata())
 
-                while True:
-                    msg = await self._receiver.get_message()
-                    if isinstance(msg, DanmuMsg):
-                        await writer.write_danmu(self._make_danmu(msg))
-                        self._calculator.submit(1)
-                    elif isinstance(msg, GiftSendMsg):
-                        if not self.record_gift_send:
-                            continue
-                        await writer.write_gift_send_record(
-                            self._make_gift_send_record(msg)
-                        )
-                    elif isinstance(msg, GuardBuyMsg):
-                        if not self.record_guard_buy:
-                            continue
-                        await writer.write_guard_buy_record(
-                            self._make_guard_buy_record(msg)
-                        )
-                    elif isinstance(msg, SuperChatMsg):
-                        if not self.record_super_chat:
-                            continue
-                        await writer.write_super_chat_record(
-                            self._make_super_chat_record(msg)
-                        )
-                    else:
-                        logger.warning('Unsupported message type:', repr(msg))
+                async for attempt in AsyncRetrying(
+                    retry=retry_if_not_exception_type((asyncio.CancelledError)),
+                    stop=stop_after_attempt(3),
+                ):
+                    with attempt:
+                        try:
+                            await self._dumping_loop(writer)
+                        except Exception as e:
+                            submit_exception(e)
+                            raise
         finally:
             logger.info(f"Danmaku file completed: '{self._path}'")
             await self._emit('danmaku_file_completed', self._path)
             logger.debug('Stopped dumping danmaku')
-            self._calculator.freeze()
+
+    async def _dumping_loop(self, writer: DanmakuWriter) -> None:
+        while True:
+            msg = await self._receiver.get_message()
+
+            if isinstance(msg, DanmuMsg):
+                await writer.write_danmu(self._make_danmu(msg))
+                self._statistics.submit(1)
+            elif isinstance(msg, GiftSendMsg):
+                if not self.record_gift_send:
+                    continue
+                record = self._make_gift_send_record(msg)
+                if not self.record_free_gifts and record.is_free_gift():
+                    continue
+                await writer.write_gift_send_record(record)
+            elif isinstance(msg, GuardBuyMsg):
+                if not self.record_guard_buy:
+                    continue
+                await writer.write_guard_buy_record(self._make_guard_buy_record(msg))
+            elif isinstance(msg, SuperChatMsg):
+                if not self.record_super_chat:
+                    continue
+                await writer.write_super_chat_record(self._make_super_chat_record(msg))
+            else:
+                logger.warning('Unsupported message type:', repr(msg))
+
+            if self._stream_recording_interrupted:
+                logger.debug(
+                    f'Last message before stream recording interrupted: {repr(msg)}'
+                )
+                async with self._stream_recording_recovered:
+                    await self._stream_recording_recovered.wait()
 
     def _make_metadata(self) -> Metadata:
         return Metadata(
@@ -235,10 +283,10 @@ class DanmakuDumper(
             ts=self._calc_stime(msg.timestamp * 1000),
             uid=msg.uid,
             user=msg.uname,
-            price=msg.price,
+            price=msg.price * msg.rate,
             time=msg.time,
             message=msg.message,
         )
 
     def _calc_stime(self, timestamp: int) -> float:
-        return max((timestamp - self._record_start_time * 1000), 0) / 1000
+        return (max(timestamp - self._timebase, 0) + self._delta) / 1000
