@@ -1,28 +1,29 @@
 from __future__ import annotations
+
+import asyncio
 import logging
 from datetime import datetime
 from typing import Iterator, Optional
 
-import aiohttp
-import aiofiles
 import humanize
-from tenacity import retry, wait_fixed, stop_after_attempt
 
-from .danmaku_receiver import DanmakuReceiver
+from blrec.bili.danmaku_client import DanmakuClient
+from blrec.bili.live import Live
+from blrec.bili.live_monitor import LiveEventListener, LiveMonitor
+from blrec.bili.models import RoomInfo
+from blrec.bili.typing import QualityNumber, StreamFormat
+from blrec.event.event_emitter import EventEmitter, EventListener
+from blrec.flv.operators import MetaData, StreamProfile
+from blrec.logging.room_id import aio_task_with_room_id
+from blrec.setting.typing import RecordingMode
+from blrec.utils.mixins import AsyncStoppableMixin
+
+from .cover_downloader import CoverDownloader, CoverSaveStrategy
 from .danmaku_dumper import DanmakuDumper, DanmakuDumperEventListener
+from .danmaku_receiver import DanmakuReceiver
+from .raw_danmaku_dumper import RawDanmakuDumper, RawDanmakuDumperEventListener
 from .raw_danmaku_receiver import RawDanmakuReceiver
-from .raw_danmaku_dumper import RawDanmakuDumper
 from .stream_recorder import StreamRecorder, StreamRecorderEventListener
-from ..event.event_emitter import EventListener, EventEmitter
-from ..bili.live import Live
-from ..bili.models import RoomInfo
-from ..bili.danmaku_client import DanmakuClient
-from ..bili.live_monitor import LiveMonitor, LiveEventListener
-from ..bili.typing import QualityNumber
-from ..utils.mixins import AsyncStoppableMixin
-from ..path import cover_path
-from ..logging.room_id import aio_task_with_room_id
-
 
 __all__ = 'RecorderEventListener', 'Recorder'
 
@@ -40,26 +41,33 @@ class RecorderEventListener(EventListener):
     async def on_recording_cancelled(self, recorder: Recorder) -> None:
         ...
 
-    async def on_video_file_created(
-        self, path: str, record_start_time: int
+    async def on_video_file_created(self, recorder: Recorder, path: str) -> None:
+        ...
+
+    async def on_video_file_completed(self, recorder: Recorder, path: str) -> None:
+        ...
+
+    async def on_danmaku_file_created(self, recorder: Recorder, path: str) -> None:
+        ...
+
+    async def on_danmaku_file_completed(self, recorder: Recorder, path: str) -> None:
+        ...
+
+    async def on_raw_danmaku_file_created(self, recorder: Recorder, path: str) -> None:
+        ...
+
+    async def on_raw_danmaku_file_completed(
+        self, recorder: Recorder, path: str
     ) -> None:
         ...
 
-    async def on_video_file_completed(self, path: str) -> None:
-        ...
-
-    async def on_danmaku_file_created(self, path: str) -> None:
-        ...
-
-    async def on_danmaku_file_completed(self, path: str) -> None:
-        ...
-
 
 class Recorder(
     EventEmitter[RecorderEventListener],
     LiveEventListener,
     AsyncStoppableMixin,
     DanmakuDumperEventListener,
+    RawDanmakuDumperEventListener,
     StreamRecorderEventListener,
 ):
     def __init__(
@@ -70,32 +78,42 @@ class Recorder(
         out_dir: str,
         path_template: str,
         *,
+        stream_format: StreamFormat = 'flv',
+        recording_mode: RecordingMode = 'standard',
+        quality_number: QualityNumber = 10000,
+        fmp4_stream_timeout: int = 10,
         buffer_size: Optional[int] = None,
         read_timeout: Optional[int] = None,
         disconnection_timeout: Optional[int] = None,
+        filesize_limit: int = 0,
+        duration_limit: int = 0,
         danmu_uname: bool = False,
         record_gift_send: bool = False,
+        record_free_gifts: bool = False,
         record_guard_buy: bool = False,
         record_super_chat: bool = False,
         save_cover: bool = False,
+        cover_save_strategy: CoverSaveStrategy = CoverSaveStrategy.DEFAULT,
         save_raw_danmaku: bool = False,
-        filesize_limit: int = 0,
-        duration_limit: int = 0,
     ) -> None:
         super().__init__()
 
         self._live = live
         self._danmaku_client = danmaku_client
         self._live_monitor = live_monitor
-        self.save_cover = save_cover
         self.save_raw_danmaku = save_raw_danmaku
 
         self._recording: bool = False
+        self._stream_available: bool = False
 
         self._stream_recorder = StreamRecorder(
             self._live,
             out_dir=out_dir,
             path_template=path_template,
+            stream_format=stream_format,
+            recording_mode=recording_mode,
+            quality_number=quality_number,
+            fmp4_stream_timeout=fmp4_stream_timeout,
             buffer_size=buffer_size,
             read_timeout=read_timeout,
             disconnection_timeout=disconnection_timeout,
@@ -110,19 +128,46 @@ class Recorder(
             self._danmaku_receiver,
             danmu_uname=danmu_uname,
             record_gift_send=record_gift_send,
+            record_free_gifts=record_free_gifts,
             record_guard_buy=record_guard_buy,
             record_super_chat=record_super_chat,
         )
         self._raw_danmaku_receiver = RawDanmakuReceiver(danmaku_client)
         self._raw_danmaku_dumper = RawDanmakuDumper(
-            self._stream_recorder,
-            self._raw_danmaku_receiver,
+            self._live, self._stream_recorder, self._raw_danmaku_receiver
         )
 
+        self._cover_downloader = CoverDownloader(
+            self._live,
+            self._stream_recorder,
+            save_cover=save_cover,
+            cover_save_strategy=cover_save_strategy,
+        )
+
+    @property
+    def live(self) -> Live:
+        return self._live
+
     @property
     def recording(self) -> bool:
         return self._recording
 
+    @property
+    def stream_format(self) -> StreamFormat:
+        return self._stream_recorder.stream_format
+
+    @stream_format.setter
+    def stream_format(self, value: StreamFormat) -> None:
+        self._stream_recorder.stream_format = value
+
+    @property
+    def recording_mode(self) -> RecordingMode:
+        return self._stream_recorder.recording_mode
+
+    @recording_mode.setter
+    def recording_mode(self, value: RecordingMode) -> None:
+        self._stream_recorder.recording_mode = value
+
     @property
     def quality_number(self) -> QualityNumber:
         return self._stream_recorder.quality_number
@@ -132,7 +177,19 @@ class Recorder(
         self._stream_recorder.quality_number = value
 
     @property
-    def real_quality_number(self) -> QualityNumber:
+    def fmp4_stream_timeout(self) -> int:
+        return self._stream_recorder.fmp4_stream_timeout
+
+    @fmp4_stream_timeout.setter
+    def fmp4_stream_timeout(self, value: int) -> None:
+        self._stream_recorder.fmp4_stream_timeout = value
+
+    @property
+    def real_stream_format(self) -> Optional[StreamFormat]:
+        return self._stream_recorder.real_stream_format
+
+    @property
+    def real_quality_number(self) -> Optional[QualityNumber]:
         return self._stream_recorder.real_quality_number
 
     @property
@@ -175,6 +232,14 @@ class Recorder(
     def record_gift_send(self, value: bool) -> None:
         self._danmaku_dumper.record_gift_send = value
 
+    @property
+    def record_free_gifts(self) -> bool:
+        return self._danmaku_dumper.record_free_gifts
+
+    @record_free_gifts.setter
+    def record_free_gifts(self, value: bool) -> None:
+        self._danmaku_dumper.record_free_gifts = value
+
     @property
     def record_guard_buy(self) -> bool:
         return self._danmaku_dumper.record_guard_buy
@@ -192,20 +257,52 @@ class Recorder(
         self._danmaku_dumper.record_super_chat = value
 
     @property
-    def elapsed(self) -> float:
-        return self._stream_recorder.elapsed
+    def save_cover(self) -> bool:
+        return self._cover_downloader.save_cover
+
+    @save_cover.setter
+    def save_cover(self, value: bool) -> None:
+        self._cover_downloader.save_cover = value
 
     @property
-    def data_count(self) -> int:
-        return self._stream_recorder.data_count
+    def cover_save_strategy(self) -> CoverSaveStrategy:
+        return self._cover_downloader.cover_save_strategy
+
+    @cover_save_strategy.setter
+    def cover_save_strategy(self, value: CoverSaveStrategy) -> None:
+        self._cover_downloader.cover_save_strategy = value
 
     @property
-    def data_rate(self) -> float:
-        return self._stream_recorder.data_rate
+    def stream_url(self) -> str:
+        return self._stream_recorder.stream_url
 
     @property
-    def danmu_count(self) -> int:
-        return self._danmaku_dumper.danmu_count
+    def stream_host(self) -> str:
+        return self._stream_recorder.stream_host
+
+    @property
+    def dl_total(self) -> int:
+        return self._stream_recorder.dl_total
+
+    @property
+    def dl_rate(self) -> float:
+        return self._stream_recorder.dl_rate
+
+    @property
+    def rec_elapsed(self) -> float:
+        return self._stream_recorder.rec_elapsed
+
+    @property
+    def rec_total(self) -> int:
+        return self._stream_recorder.rec_total
+
+    @property
+    def rec_rate(self) -> float:
+        return self._stream_recorder.rec_rate
+
+    @property
+    def danmu_total(self) -> int:
+        return self._danmaku_dumper.danmu_total
 
     @property
     def danmu_rate(self) -> float:
@@ -243,24 +340,17 @@ class Recorder(
     def duration_limit(self, value: int) -> None:
         self._stream_recorder.duration_limit = value
 
-    async def _do_start(self) -> None:
-        self._live_monitor.add_listener(self)
-        self._danmaku_dumper.add_listener(self)
-        self._stream_recorder.add_listener(self)
-        logger.debug('Started recorder')
+    @property
+    def recording_path(self) -> Optional[str]:
+        return self._stream_recorder.recording_path
 
-        self._print_live_info()
-        if self._live.is_living():
-            await self._start_recording(stream_available=True)
-        else:
-            self._print_waiting_message()
+    @property
+    def metadata(self) -> Optional[MetaData]:
+        return self._stream_recorder.metadata
 
-    async def _do_stop(self) -> None:
-        await self._stop_recording()
-        self._live_monitor.remove_listener(self)
-        self._danmaku_dumper.remove_listener(self)
-        self._stream_recorder.remove_listener(self)
-        logger.debug('Stopped recorder')
+    @property
+    def stream_profile(self) -> StreamProfile:
+        return self._stream_recorder.stream_profile
 
     def get_recording_files(self) -> Iterator[str]:
         if self._stream_recorder.recording_path is not None:
@@ -287,43 +377,72 @@ class Recorder(
 
     async def on_live_ended(self, live: Live) -> None:
         logger.info('The live has ended')
+        await asyncio.sleep(3)
+        self._stream_available = False
+        self._stream_recorder.stream_available_time = None
         await self._stop_recording()
         self._print_waiting_message()
 
     async def on_live_stream_available(self, live: Live) -> None:
         logger.debug('The live stream becomes available')
+        self._stream_available = True
+        self._stream_recorder.stream_available_time = await live.get_timestamp()
         await self._stream_recorder.start()
 
     async def on_live_stream_reset(self, live: Live) -> None:
         logger.warning('The live stream has been reset')
         if not self._recording:
-            await self._start_recording(stream_available=True)
+            await self._start_recording()
 
     async def on_room_changed(self, room_info: RoomInfo) -> None:
         self._print_changed_room_info(room_info)
         self._stream_recorder.update_progress_bar_info()
 
-    async def on_video_file_created(
-        self, path: str, record_start_time: int
-    ) -> None:
-        await self._emit('video_file_created', path, record_start_time)
+    async def on_video_file_created(self, path: str, record_start_time: int) -> None:
+        await self._emit('video_file_created', self, path)
 
     async def on_video_file_completed(self, path: str) -> None:
-        await self._emit('video_file_completed', path)
-        if self.save_cover:
-            await self._save_cover_image(path)
+        await self._emit('video_file_completed', self, path)
 
     async def on_danmaku_file_created(self, path: str) -> None:
-        await self._emit('danmaku_file_created', path)
+        await self._emit('danmaku_file_created', self, path)
 
     async def on_danmaku_file_completed(self, path: str) -> None:
-        await self._emit('danmaku_file_completed', path)
+        await self._emit('danmaku_file_completed', self, path)
 
-    async def on_stream_recording_stopped(self) -> None:
-        logger.debug('Stream recording stopped')
+    async def on_raw_danmaku_file_created(self, path: str) -> None:
+        await self._emit('raw_danmaku_file_created', self, path)
+
+    async def on_raw_danmaku_file_completed(self, path: str) -> None:
+        await self._emit('raw_danmaku_file_completed', self, path)
+
+    async def on_stream_recording_completed(self) -> None:
+        logger.debug('Stream recording completed')
         await self._stop_recording()
 
-    async def _start_recording(self, stream_available: bool = False) -> None:
+    async def _do_start(self) -> None:
+        self._live_monitor.add_listener(self)
+        self._danmaku_dumper.add_listener(self)
+        self._raw_danmaku_dumper.add_listener(self)
+        self._stream_recorder.add_listener(self)
+        logger.debug('Started recorder')
+
+        self._print_live_info()
+        if self._live.is_living():
+            self._stream_available = True
+            await self._start_recording()
+        else:
+            self._print_waiting_message()
+
+    async def _do_stop(self) -> None:
+        await self._stop_recording()
+        self._live_monitor.remove_listener(self)
+        self._danmaku_dumper.remove_listener(self)
+        self._raw_danmaku_dumper.remove_listener(self)
+        self._stream_recorder.remove_listener(self)
+        logger.debug('Stopped recorder')
+
+    async def _start_recording(self) -> None:
         if self._recording:
             return
         self._recording = True
@@ -333,10 +452,13 @@ class Recorder(
             self._raw_danmaku_receiver.start()
         self._danmaku_dumper.enable()
         self._danmaku_receiver.start()
+        self._cover_downloader.enable()
 
         await self._prepare()
-        if stream_available:
+        if self._stream_available:
             await self._stream_recorder.start()
+        else:
+            asyncio.create_task(self._guard())
 
         logger.info('Started recording')
         await self._emit('recording_started', self)
@@ -352,6 +474,7 @@ class Recorder(
             self._raw_danmaku_receiver.stop()
         self._danmaku_dumper.disable()
         self._danmaku_receiver.stop()
+        self._cover_downloader.disable()
 
         if self._stopped:
             logger.info('Recording Cancelled')
@@ -366,23 +489,28 @@ class Recorder(
         self._danmaku_dumper.clear_files()
         self._stream_recorder.clear_files()
 
-    @retry(wait=wait_fixed(1), stop=stop_after_attempt(3))
     @aio_task_with_room_id
-    async def _save_cover_image(self, video_path: str) -> None:
+    async def _guard(self, timeout: float = 60) -> None:
+        await asyncio.sleep(timeout)
+        if not self._recording:
+            return
+        if self._stream_available:
+            return
+        logger.debug(
+            f'Stream not available in {timeout} seconds, the event maybe lost.'
+        )
+
         await self._live.update_info()
-        async with aiohttp.ClientSession(raise_for_status=True) as session:
-            try:
-                url = self._live.room_info.cover
-                async with session.get(url) as response:
-                    ext = url.rsplit('.', 1)[-1]
-                    path = cover_path(video_path, ext)
-                    async with aiofiles.open(path, 'wb') as file:
-                        await file.write(await response.read())
-            except Exception as e:
-                logger.error(f'Failed to save cover image: {repr(e)}')
-                raise
-            else:
-                logger.info(f'Saved cover image: {path}')
+        if self._live.is_living():
+            logger.debug('The live is living now')
+            self._stream_available = True
+            if self._stream_recorder.stopped:
+                await self._stream_recorder.start()
+        else:
+            logger.debug('The live has ended before streaming')
+            self._stream_available = False
+            if not self._stream_recorder.stopped:
+                await self.stop()
 
     def _print_waiting_message(self) -> None:
         logger.info('Waiting... until the live starts')
@@ -392,9 +520,7 @@ class Recorder(
         user_info = self._live.user_info
 
         if room_info.live_start_time > 0:
-            live_start_time = str(
-                datetime.fromtimestamp(room_info.live_start_time)
-            )
+            live_start_time = str(datetime.fromtimestamp(room_info.live_start_time))
         else:
             live_start_time = 'NULL'
 
