@@ -3,13 +3,17 @@ from .notifiers import (
     MessageNotifier,
     EmailNotifier,
     ServerchanNotifier,
+    PushdeerNotifier,
     PushplusNotifier,
+    TelegramNotifier,
 )
 from .providers import (
     MessagingProvider,
     EmailService,
     Serverchan,
+    Pushdeer,
     Pushplus,
+    Telegram,
 )
 
 
@@ -17,11 +21,15 @@ __all__ = (
     'MessagingProvider',
     'EmailService',
     'Serverchan',
+    'Pushdeer',
     'Pushplus',
+    'Telegram',
 
     'Notifier',
     'MessageNotifier',
     'EmailNotifier',
     'ServerchanNotifier',
+    'PushdeerNotifier',
     'PushplusNotifier',
+    'TelegramNotifier',
 )
