@@ -1,43 +1,40 @@
 from __future__ import annotations
+
+import logging
 import os
 import re
-import logging
-from typing_extensions import Annotated
-from typing import (
-    ClassVar,
-    Collection,
-    Final,
-    List,
-    Optional,
-    TypeVar,
-)
-
+from typing import ClassVar, Collection, Final, List, Optional, TypeVar
 
 import toml
 from pydantic import BaseModel as PydanticBaseModel
-from pydantic import Field, BaseSettings, validator, PrivateAttr, DirectoryPath
-from pydantic.networks import HttpUrl, EmailStr
+from pydantic import BaseSettings, Field, PrivateAttr, validator
+from pydantic.networks import EmailStr, HttpUrl
+from typing_extensions import Annotated
 
-import typer
-
-from ..bili.typing import QualityNumber
-from ..postprocess import DeleteStrategy
-from ..logging.typing import LOG_LEVEL
-from ..utils.string import camel_case
-from ..path.helpers import file_exists, create_file
+from blrec.bili.typing import QualityNumber, StreamFormat
+from blrec.core.cover_downloader import CoverSaveStrategy
+from blrec.logging.typing import LOG_LEVEL
+from blrec.postprocess import DeleteStrategy
+from blrec.utils.string import camel_case
 
+from .typing import (
+    EmailMessageType,
+    PushdeerMessageType,
+    PushplusMessageType,
+    RecordingMode,
+    ServerchanMessageType,
+    TelegramMessageType,
+)
 
 logger = logging.getLogger(__name__)
 
 
 __all__ = (
-    'DEFAULT_SETTINGS_PATH',
-
+    'DEFAULT_SETTINGS_FILE',
     'EnvSettings',
     'Settings',
     'SettingsIn',
     'SettingsOut',
-
     'HeaderOptions',
     'HeaderSettings',
     'DanmakuOptions',
@@ -46,7 +43,6 @@ __all__ = (
     'RecorderSettings',
     'PostprocessingSettings',
     'PostprocessingOptions',
-
     'TaskOptions',
     'TaskSettings',
     'OutputSettings',
@@ -54,39 +50,38 @@ __all__ = (
     'SpaceSettings',
     'EmailSettings',
     'ServerchanSettings',
+    'PushdeerSettings',
     'PushplusSettings',
+    'TelegramSettings',
     'NotifierSettings',
     'NotificationSettings',
+    'EmailMessageTemplateSettings',
+    'ServerchanMessageTemplateSettings',
+    'PushdeerMessageTemplateSettings',
+    'PushplusMessageTemplateSettings',
+    'TelegramMessageTemplateSettings',
     'EmailNotificationSettings',
     'ServerchanNotificationSettings',
+    'PushdeerNotificationSettings',
     'PushplusNotificationSettings',
+    'TelegramNotificationSettings',
     'WebHookSettings',
 )
 
 
-DEFAULT_SETTINGS_PATH: Final[str] = '~/.blrec/settings.toml'
-
-
-def settings_file_factory() -> str:
-    path = os.path.abspath(os.path.expanduser(DEFAULT_SETTINGS_PATH))
-    if not file_exists(path):
-        create_file(path)
-        typer.secho(
-            f"Created setting file: '{path}'",
-            fg=typer.colors.BRIGHT_MAGENTA,
-            bold=True,
-        )
-    return path
+DEFAULT_OUT_DIR: Final[str] = os.environ.get('DEFAULT_OUT_DIR', '.')
+DEFAULT_LOG_DIR: Final[str] = os.environ.get('DEFAULT_LOG_DIR', '~/.blrec/logs/')
+DEFAULT_SETTINGS_FILE: Final[str] = os.environ.get(
+    'DEFAULT_SETTINGS_FILE', '~/.blrec/settings.toml'
+)
 
 
 class EnvSettings(BaseSettings):
-    settings_file: Annotated[
-        str, Field(env='config', default_factory=settings_file_factory)
-    ]
+    settings_file: Annotated[str, Field(env='config')] = DEFAULT_SETTINGS_FILE
     out_dir: Optional[str] = None
+    log_dir: Optional[str] = None
     api_key: Annotated[
-        Optional[str],
-        Field(min_length=8, max_length=80, regex=r'[a-zA-Z\d\-]{8,80}'),
+        Optional[str], Field(min_length=8, max_length=80, regex=r'[a-zA-Z\d\-]{8,80}'),
     ] = None
 
     class Config:
@@ -98,7 +93,6 @@ _V = TypeVar('_V')
 
 class BaseModel(PydanticBaseModel):
     class Config:
-        extra = 'forbid'
         validate_assignment = True
         anystr_strip_whitespace = True
         allow_population_by_field_name = True
@@ -108,9 +102,7 @@ class BaseModel(PydanticBaseModel):
             return camel_case(string)
 
     @staticmethod
-    def _validate_with_collection(
-        value: _V, allowed_values: Collection[_V]
-    ) -> None:
+    def _validate_with_collection(value: _V, allowed_values: Collection[_V]) -> None:
         if value not in allowed_values:
             raise ValueError(
                 f'the value {value} does not be allowed, '
@@ -124,15 +116,18 @@ class HeaderOptions(BaseModel):
 
 
 class HeaderSettings(HeaderOptions):
-    user_agent: str = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) ' \
-        'AppleWebKit/537.36 (KHTML, like Gecko) ' \
+    user_agent: str = (
+        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
+        'AppleWebKit/537.36 (KHTML, like Gecko) '
         'Chrome/89.0.4389.114 Safari/537.36'
+    )
     cookie: str = ''
 
 
 class DanmakuOptions(BaseModel):
     danmu_uname: Optional[bool]
     record_gift_send: Optional[bool]
+    record_free_gifts: Optional[bool]
     record_guard_buy: Optional[bool]
     record_super_chat: Optional[bool]
     save_raw_danmaku: Optional[bool]
@@ -141,19 +136,31 @@ class DanmakuOptions(BaseModel):
 class DanmakuSettings(DanmakuOptions):
     danmu_uname: bool = False
     record_gift_send: bool = True
+    record_free_gifts: bool = True
     record_guard_buy: bool = True
     record_super_chat: bool = True
     save_raw_danmaku: bool = False
 
 
 class RecorderOptions(BaseModel):
+    stream_format: Optional[StreamFormat]
+    recording_mode: Optional[RecordingMode]
     quality_number: Optional[QualityNumber]
+    fmp4_stream_timeout: Optional[int]
     read_timeout: Optional[int]  # seconds
     disconnection_timeout: Optional[int]  # seconds
     buffer_size: Annotated[  # bytes
-        Optional[int], Field(ge=4096, le=1024 ** 2 * 512, multiple_of=2)
+        Optional[int], Field(ge=4096, le=1024**2 * 512, multiple_of=2)
     ]
     save_cover: Optional[bool]
+    cover_save_strategy: Optional[CoverSaveStrategy]
+
+    @validator('fmp4_stream_timeout')
+    def _validate_fmp4_stream_timeout(cls, v: Optional[int]) -> Optional[int]:
+        if v is not None:
+            allowed_values = frozenset((3, 5, 10, 30, 60, 180, 300, 600))
+            cls._validate_with_collection(v, allowed_values)
+        return v
 
     @validator('read_timeout')
     def _validate_read_timeout(cls, value: Optional[int]) -> Optional[int]:
@@ -163,9 +170,7 @@ class RecorderOptions(BaseModel):
         return value
 
     @validator('disconnection_timeout')
-    def _validate_disconnection_timeout(
-        cls, value: Optional[int]
-    ) -> Optional[int]:
+    def _validate_disconnection_timeout(cls, value: Optional[int]) -> Optional[int]:
         if value is not None:
             allowed_values = frozenset(60 * i for i in (3, 5, 10, 15, 20, 30))
             cls._validate_with_collection(value, allowed_values)
@@ -173,13 +178,17 @@ class RecorderOptions(BaseModel):
 
 
 class RecorderSettings(RecorderOptions):
+    stream_format: StreamFormat = 'flv'
+    recording_mode: RecordingMode = 'standard'
     quality_number: QualityNumber = 20000  # 4K, the highest quality.
+    fmp4_stream_timeout: int = 10
     read_timeout: int = 3
     disconnection_timeout: int = 600
     buffer_size: Annotated[
-        int, Field(ge=4096, le=1024 ** 2 * 512, multiple_of=2)
+        int, Field(ge=4096, le=1024**2 * 512, multiple_of=2)
     ] = 8192
     save_cover: bool = False
+    cover_save_strategy: CoverSaveStrategy = CoverSaveStrategy.DEFAULT
 
 
 class PostprocessingOptions(BaseModel):
@@ -233,23 +242,33 @@ class OutputOptions(BaseModel):
 
     @validator('filesize_limit')
     def _validate_filesize_limit(cls, value: Optional[int]) -> Optional[int]:
-        # allowed 1 ~ 20 GB, 0 indicates not limit.
+        # file size in bytes, 0 indicates not limit。
         if value is not None:
-            allowed_values = frozenset(1024 ** 3 * i for i in range(0, 21))
-            cls._validate_with_collection(value, allowed_values)
+            if not (0 <= value <= 1073731086581):  # 1073731086581(999.99 GB)
+                raise ValueError(
+                    'The filesize limit must be in the range of 0 to 1073731086581'
+                )
         return value
 
     @validator('duration_limit')
     def _validate_duration_limit(cls, value: Optional[int]) -> Optional[int]:
-        # allowed 1 ~ 24 hours, 0 indicates not limit.
+        # duration in seconds, 0 indicates not limit。
         if value is not None:
-            allowed_values = frozenset(3600 * i for i in range(0, 25))
-            cls._validate_with_collection(value, allowed_values)
+            if not (0 <= value <= 359999):  # 359999(99:59:59)
+                raise ValueError(
+                    'The duration limit must be in the range of 0 to 359999'
+                )
         return value
 
 
+def out_dir_factory() -> str:
+    path = os.path.normpath(os.path.expanduser(DEFAULT_OUT_DIR))
+    os.makedirs(path, exist_ok=True)
+    return path
+
+
 class OutputSettings(OutputOptions):
-    out_dir: Annotated[str, DirectoryPath] = '.'
+    out_dir: Annotated[str, Field(default_factory=out_dir_factory)]
     path_template: str = (
         '{roomid} - {uname}/'
         'blive_{roomid}_{year}-{month}-{day}-{hour}{minute}{second}'
@@ -273,11 +292,11 @@ class TaskOptions(BaseModel):
 
     @classmethod
     def from_settings(cls, settings: TaskSettings) -> TaskOptions:
-        return cls(**settings.dict(
-            include={
-                'output', 'header', 'danmaku', 'recorder', 'postprocessing'
-            }
-        ))
+        return cls(
+            **settings.dict(
+                include={'output', 'header', 'danmaku', 'recorder', 'postprocessing'}
+            )
+        )
 
 
 class TaskSettings(TaskOptions):
@@ -287,28 +306,43 @@ class TaskSettings(TaskOptions):
     enable_recorder: bool = True
 
 
+def log_dir_factory() -> str:
+    path = os.path.normpath(os.path.expanduser(DEFAULT_LOG_DIR))
+    os.makedirs(path, exist_ok=True)
+    return path
+
+
 class LoggingSettings(BaseModel):
+    log_dir: Annotated[str, Field(default_factory=log_dir_factory)]
     console_log_level: LOG_LEVEL = 'INFO'
     max_bytes: Annotated[
-        int, Field(ge=1024 ** 2, le=1024 ** 2 * 10, multiple_of=1024 ** 2)
-    ] = 1024 ** 2 * 10  # allowed 1 ~ 10 MB
+        int, Field(ge=1024**2, le=1024**2 * 10, multiple_of=1024**2)
+    ] = (
+        1024**2 * 10
+    )  # allowed 1 ~ 10 MB
     backup_count: Annotated[int, Field(ge=1, le=30)] = 30
 
+    @validator('log_dir')
+    def _validate_dir(cls, path: str) -> str:
+        if not os.path.isdir(os.path.expanduser(path)):
+            raise ValueError(f"'{path}' not a directory")
+        return path
+
 
 class SpaceSettings(BaseModel):
     check_interval: int = 60  # 1 minutes
-    space_threshold: int = 1024 ** 3  # 1 GB
+    space_threshold: int = 1024**3  # 1 GB
     recycle_records: bool = False
 
     @validator('check_interval')
     def _validate_interval(cls, value: int) -> int:
-        allowed_values = frozenset(60 * i for i in (1, 3, 5, 10))
+        allowed_values = frozenset((0, 10, 30, *(60 * i for i in (1, 3, 5, 10))))
         cls._validate_with_collection(value, allowed_values)
         return value
 
     @validator('space_threshold')
     def _validate_threshold(cls, value: int) -> int:
-        allowed_values = frozenset(1024 ** 3 * i for i in (1, 3, 5, 10))
+        allowed_values = frozenset(1024**3 * i for i in (1, 3, 5, 10, 20))
         cls._validate_with_collection(value, allowed_values)
         return value
 
@@ -331,6 +365,25 @@ class ServerchanSettings(BaseModel):
         return value
 
 
+class PushdeerSettings(BaseModel):
+    server: str = ''
+    pushkey: str = ''
+
+    @validator('server')
+    def _validate_server(cls, value: str) -> str:
+        if value != '' and not re.fullmatch(r'https?://.+', value):
+            raise ValueError('server is invalid')
+        return value
+
+    @validator('pushkey')
+    def _validate_pushkey(cls, value: str) -> str:
+        if value != '' and not re.fullmatch(
+            r'PDU\d+T[a-zA-Z\d]{32}(,PDU\d+T[a-zA-Z\d]{32}){0,99}', value
+        ):
+            raise ValueError('pushkey is invalid')
+        return value
+
+
 class PushplusSettings(BaseModel):
     token: str = ''
     topic: str = ''
@@ -342,6 +395,23 @@ class PushplusSettings(BaseModel):
         return value
 
 
+class TelegramSettings(BaseModel):
+    token: str = ''
+    chatid: str = ''
+
+    @validator('token')
+    def _validate_token(cls, value: str) -> str:
+        if value != '' and not re.fullmatch(r'[0-9]{8,10}:[a-zA-Z0-9_-]{35}', value):
+            raise ValueError('token is invalid')
+        return value
+
+    @validator('chatid')
+    def _validate_chatid(cls, value: str) -> str:
+        if value != '' and not re.fullmatch(r'(-|[0-9]){0,}', value):
+            raise ValueError('chatid is invalid')
+        return value
+
+
 class NotifierSettings(BaseModel):
     enabled: bool = False
 
@@ -353,20 +423,134 @@ class NotificationSettings(BaseModel):
     notify_space: bool = True
 
 
+class MessageTemplateSettings(BaseModel):
+    began_message_type: str
+    began_message_title: str
+    began_message_content: str
+    ended_message_type: str
+    ended_message_title: str
+    ended_message_content: str
+    space_message_type: str
+    space_message_title: str
+    space_message_content: str
+    error_message_type: str
+    error_message_title: str
+    error_message_content: str
+
+
+class EmailMessageTemplateSettings(MessageTemplateSettings):
+    began_message_type: EmailMessageType = 'html'
+    began_message_title: str = ''
+    began_message_content: str = ''
+    ended_message_type: EmailMessageType = 'html'
+    ended_message_title: str = ''
+    ended_message_content: str = ''
+    space_message_type: EmailMessageType = 'html'
+    space_message_title: str = ''
+    space_message_content: str = ''
+    error_message_type: EmailMessageType = 'html'
+    error_message_title: str = ''
+    error_message_content: str = ''
+
+
+class ServerchanMessageTemplateSettings(MessageTemplateSettings):
+    began_message_type: ServerchanMessageType = 'markdown'
+    began_message_title: str = ''
+    began_message_content: str = ''
+    ended_message_type: ServerchanMessageType = 'markdown'
+    ended_message_title: str = ''
+    ended_message_content: str = ''
+    space_message_type: ServerchanMessageType = 'markdown'
+    space_message_title: str = ''
+    space_message_content: str = ''
+    error_message_type: ServerchanMessageType = 'markdown'
+    error_message_title: str = ''
+    error_message_content: str = ''
+
+
+class PushdeerMessageTemplateSettings(MessageTemplateSettings):
+    began_message_type: PushdeerMessageType = 'markdown'
+    began_message_title: str = ''
+    began_message_content: str = ''
+    ended_message_type: PushdeerMessageType = 'markdown'
+    ended_message_title: str = ''
+    ended_message_content: str = ''
+    space_message_type: PushdeerMessageType = 'markdown'
+    space_message_title: str = ''
+    space_message_content: str = ''
+    error_message_type: PushdeerMessageType = 'markdown'
+    error_message_title: str = ''
+    error_message_content: str = ''
+
+
+class PushplusMessageTemplateSettings(MessageTemplateSettings):
+    began_message_type: PushplusMessageType = 'markdown'
+    began_message_title: str = ''
+    began_message_content: str = ''
+    ended_message_type: PushplusMessageType = 'markdown'
+    ended_message_title: str = ''
+    ended_message_content: str = ''
+    space_message_type: PushplusMessageType = 'markdown'
+    space_message_title: str = ''
+    space_message_content: str = ''
+    error_message_type: PushplusMessageType = 'markdown'
+    error_message_title: str = ''
+    error_message_content: str = ''
+
+
+class TelegramMessageTemplateSettings(MessageTemplateSettings):
+    began_message_type: TelegramMessageType = 'html'
+    began_message_title: str = ''
+    began_message_content: str = ''
+    ended_message_type: TelegramMessageType = 'html'
+    ended_message_title: str = ''
+    ended_message_content: str = ''
+    space_message_type: TelegramMessageType = 'html'
+    space_message_title: str = ''
+    space_message_content: str = ''
+    error_message_type: TelegramMessageType = 'html'
+    error_message_title: str = ''
+    error_message_content: str = ''
+
+
 class EmailNotificationSettings(
-    EmailSettings, NotifierSettings, NotificationSettings
+    EmailSettings, NotifierSettings, NotificationSettings, EmailMessageTemplateSettings
 ):
     pass
 
 
 class ServerchanNotificationSettings(
-    ServerchanSettings, NotifierSettings, NotificationSettings
+    ServerchanSettings,
+    NotifierSettings,
+    NotificationSettings,
+    ServerchanMessageTemplateSettings,
+):
+    pass
+
+
+class PushdeerNotificationSettings(
+    PushdeerSettings,
+    NotifierSettings,
+    NotificationSettings,
+    PushplusMessageTemplateSettings,
 ):
     pass
 
 
 class PushplusNotificationSettings(
-    PushplusSettings, NotifierSettings, NotificationSettings
+    PushplusSettings,
+    NotifierSettings,
+    NotificationSettings,
+    PushplusMessageTemplateSettings,
+):
+    pass
+
+
+class TelegramNotificationSettings(
+    TelegramSettings,
+    NotifierSettings,
+    NotificationSettings,
+    TelegramMessageTemplateSettings,
 ):
     pass
 
@@ -375,8 +559,17 @@ class WebHookEventSettings(BaseModel):
     live_began: bool = True
     live_ended: bool = True
     room_change: bool = True
+    recording_started: bool = True
+    recording_finished: bool = True
+    recording_cancelled: bool = True
+    video_file_created: bool = True
+    video_file_completed: bool = True
+    danmaku_file_created: bool = True
+    danmaku_file_completed: bool = True
+    raw_danmaku_file_created: bool = True
+    raw_danmaku_file_completed: bool = True
+    video_postprocessing_completed: bool = True
     space_no_enough: bool = True
-    file_completed: bool = True
     error_occurred: bool = True
 
 
@@ -392,18 +585,20 @@ class Settings(BaseModel):
     version: str = '1.0'
 
     tasks: Annotated[List[TaskSettings], Field(max_items=100)] = []
-    output: OutputSettings = OutputSettings()
-    logging: LoggingSettings = LoggingSettings()
+    output: OutputSettings = OutputSettings()  # type: ignore
+    logging: LoggingSettings = LoggingSettings()  # type: ignore
     header: HeaderSettings = HeaderSettings()
     danmaku: DanmakuSettings = DanmakuSettings()
     recorder: RecorderSettings = RecorderSettings()
     postprocessing: PostprocessingSettings = PostprocessingSettings()
     space: SpaceSettings = SpaceSettings()
     email_notification: EmailNotificationSettings = EmailNotificationSettings()
-    serverchan_notification: ServerchanNotificationSettings = \
+    serverchan_notification: ServerchanNotificationSettings = (
         ServerchanNotificationSettings()
-    pushplus_notification: PushplusNotificationSettings = \
-        PushplusNotificationSettings()
+    )
+    pushdeer_notification: PushdeerNotificationSettings = PushdeerNotificationSettings()
+    pushplus_notification: PushplusNotificationSettings = PushplusNotificationSettings()
+    telegram_notification: TelegramNotificationSettings = TelegramNotificationSettings()
     webhooks: Annotated[List[WebHookSettings], Field(max_items=50)] = []
 
     @classmethod
@@ -415,6 +610,8 @@ class Settings(BaseModel):
     def update_from_env_settings(self, env_settings: EnvSettings) -> None:
         if (out_dir := env_settings.out_dir) is not None:
             self.output.out_dir = out_dir
+        if (log_dir := env_settings.log_dir) is not None:
+            self.logging.log_dir = log_dir
 
     def dump(self) -> None:
         assert self._path
@@ -423,7 +620,7 @@ class Settings(BaseModel):
 
     @validator('tasks')
     def _validate_tasks(cls, tasks: List[TaskSettings]) -> List[TaskSettings]:
-        if len(tasks) >= cls._MAX_TASKS:
+        if len(tasks) > cls._MAX_TASKS:
             raise ValueError(f'Out of max tasks limits: {cls._MAX_TASKS}')
         return tasks
 
@@ -432,9 +629,7 @@ class Settings(BaseModel):
         cls, webhooks: List[WebHookSettings]
     ) -> List[WebHookSettings]:
         if len(webhooks) >= cls._MAX_WEBHOOKS:
-            raise ValueError(
-                f'Out of max webhooks limits: {cls._MAX_WEBHOOKS}'
-            )
+            raise ValueError(f'Out of max webhooks limits: {cls._MAX_WEBHOOKS}')
         return webhooks
 
 
@@ -448,7 +643,9 @@ class SettingsIn(BaseModel):
     space: Optional[SpaceSettings] = None
     email_notification: Optional[EmailNotificationSettings] = None
     serverchan_notification: Optional[ServerchanNotificationSettings] = None
+    pushdeer_notification: Optional[PushdeerNotificationSettings] = None
     pushplus_notification: Optional[PushplusNotificationSettings] = None
+    telegram_notification: Optional[TelegramNotificationSettings] = None
     webhooks: Optional[List[WebHookSettings]] = None
 
 
