@@ -1,33 +1,37 @@
-import os
 import logging
+import os
 from pathlib import Path
 from typing import Iterator, Optional
 
+from blrec.bili.danmaku_client import DanmakuClient
+from blrec.bili.live import Live
+from blrec.bili.live_monitor import LiveMonitor
+from blrec.bili.models import RoomInfo, UserInfo
+from blrec.bili.typing import QualityNumber, StreamFormat
+from blrec.core import Recorder
+from blrec.core.cover_downloader import CoverSaveStrategy
+from blrec.event.event_submitters import (
+    LiveEventSubmitter,
+    PostprocessorEventSubmitter,
+    RecorderEventSubmitter,
+)
+from blrec.flv.metadata_injection import InjectingProgress
+from blrec.flv.operators import MetaData, StreamProfile
+from blrec.logging.room_id import aio_task_with_room_id
+from blrec.postprocess import DeleteStrategy, Postprocessor, PostprocessorStatus
+from blrec.postprocess.remux import RemuxingProgress
+from blrec.setting.typing import RecordingMode
 
 from .models import (
-    TaskStatus,
-    RunningStatus,
-    VideoFileStatus,
-    VideoFileDetail,
-    DanmukuFileStatus,
     DanmakuFileDetail,
+    DanmukuFileStatus,
+    RunningStatus,
+    TaskStatus,
+    VideoFileDetail,
+    VideoFileStatus,
 )
-from ..bili.live import Live
-from ..bili.models import RoomInfo, UserInfo
-from ..bili.danmaku_client import DanmakuClient
-from ..bili.live_monitor import LiveMonitor
-from ..bili.typing import QualityNumber
-from ..core import Recorder
-from ..postprocess import Postprocessor, PostprocessorStatus, DeleteStrategy
-from ..postprocess.remuxer import RemuxProgress
-from ..flv.metadata_injector import InjectProgress
-from ..event.event_submitters import (
-    LiveEventSubmitter, PostprocessorEventSubmitter
-)
-from ..logging.room_id import aio_task_with_room_id
 
-
-__all__ = 'RecordTask',
+__all__ = ('RecordTask',)
 
 
 logger = logging.getLogger(__name__)
@@ -42,17 +46,6 @@ class RecordTask:
         path_template: str = '',
         cookie: str = '',
         user_agent: str = '',
-        danmu_uname: bool = False,
-        record_gift_send: bool = False,
-        record_guard_buy: bool = False,
-        record_super_chat: bool = False,
-        save_cover: bool = False,
-        save_raw_danmaku: bool = False,
-        buffer_size: Optional[int] = None,
-        read_timeout: Optional[int] = None,
-        disconnection_timeout: Optional[int] = None,
-        filesize_limit: int = 0,
-        duration_limit: int = 0,
         remux_to_mp4: bool = False,
         inject_extra_metadata: bool = True,
         delete_source: DeleteStrategy = DeleteStrategy.AUTO,
@@ -66,17 +59,6 @@ class RecordTask:
         self._path_template = path_template
         self._cookie = cookie
         self._user_agent = user_agent
-        self._danmu_uname = danmu_uname
-        self._record_gift_send = record_gift_send
-        self._record_guard_buy = record_guard_buy
-        self._record_super_chat = record_super_chat
-        self._save_cover = save_cover
-        self._save_raw_danmaku = save_raw_danmaku
-        self._buffer_size = buffer_size
-        self._read_timeout = read_timeout
-        self._disconnection_timeout = disconnection_timeout
-        self._filesize_limit = filesize_limit
-        self._duration_limit = duration_limit
         self._remux_to_mp4 = remux_to_mp4
         self._inject_extra_metadata = inject_extra_metadata
         self._delete_source = delete_source
@@ -124,17 +106,21 @@ class RecordTask:
             monitor_enabled=self.monitor_enabled,
             recorder_enabled=self.recorder_enabled,
             running_status=self.running_status,
-            elapsed=self._recorder.elapsed,
-            data_count=self._recorder.data_count,
-            data_rate=self._recorder.data_rate,
-            danmu_count=self._recorder.danmu_count,
+            stream_url=self._recorder.stream_url,
+            stream_host=self._recorder.stream_host,
+            dl_total=self._recorder.dl_total,
+            dl_rate=self._recorder.dl_rate,
+            rec_elapsed=self._recorder.rec_elapsed,
+            rec_total=self._recorder.rec_total,
+            rec_rate=self._recorder.rec_rate,
+            danmu_total=self._recorder.danmu_total,
             danmu_rate=self._recorder.danmu_rate,
+            real_stream_format=self._recorder.real_stream_format,
             real_quality_number=self._recorder.real_quality_number,
+            recording_path=self.recording_path,
             postprocessor_status=self._postprocessor.status,
             postprocessing_path=self._postprocessor.postprocessing_path,
-            postprocessing_progress=(
-                self._postprocessor.postprocessing_progress
-            ),
+            postprocessing_progress=(self._postprocessor.postprocessing_progress),
         )
 
     @property
@@ -164,19 +150,20 @@ class RecordTask:
                 status = VideoFileStatus.RECORDING
             elif path == self._postprocessor.postprocessing_path:
                 progress = self._postprocessor.postprocessing_progress
-                if isinstance(progress, RemuxProgress):
+                if isinstance(progress, RemuxingProgress):
                     status = VideoFileStatus.REMUXING
-                elif isinstance(progress, InjectProgress):
+                elif isinstance(progress, InjectingProgress):
                     status = VideoFileStatus.INJECTING
+                else:
+                    if self._postprocessor.remux_to_mp4:
+                        status = VideoFileStatus.REMUXING
+                    else:
+                        status = VideoFileStatus.INJECTING
             else:
                 # disabling recorder by force or stoping task by force
                 status = VideoFileStatus.BROKEN
 
-            yield VideoFileDetail(
-                path=path,
-                size=size,
-                status=status,
-            )
+            yield VideoFileDetail(path=path, size=size, status=status)
 
     @property
     def danmaku_file_details(self) -> Iterator[DanmakuFileDetail]:
@@ -201,11 +188,7 @@ class RecordTask:
                 # disabling recorder by force or stoping task by force
                 status = DanmukuFileStatus.BROKEN
 
-            yield DanmakuFileDetail(
-                path=path,
-                size=size,
-                status=status,
-            )
+            yield DanmakuFileDetail(path=path, size=size, status=status)
 
     @property
     def user_agent(self) -> str:
@@ -239,6 +222,14 @@ class RecordTask:
     def record_gift_send(self, value: bool) -> None:
         self._recorder.record_gift_send = value
 
+    @property
+    def record_free_gifts(self) -> bool:
+        return self._recorder.record_free_gifts
+
+    @record_free_gifts.setter
+    def record_free_gifts(self, value: bool) -> None:
+        self._recorder.record_free_gifts = value
+
     @property
     def record_guard_buy(self) -> bool:
         return self._recorder.record_guard_buy
@@ -263,6 +254,14 @@ class RecordTask:
     def save_cover(self, value: bool) -> None:
         self._recorder.save_cover = value
 
+    @property
+    def cover_save_strategy(self) -> CoverSaveStrategy:
+        return self._recorder.cover_save_strategy
+
+    @cover_save_strategy.setter
+    def cover_save_strategy(self, value: CoverSaveStrategy) -> None:
+        self._recorder.cover_save_strategy = value
+
     @property
     def save_raw_danmaku(self) -> bool:
         return self._recorder.save_raw_danmaku
@@ -271,6 +270,22 @@ class RecordTask:
     def save_raw_danmaku(self, value: bool) -> None:
         self._recorder.save_raw_danmaku = value
 
+    @property
+    def stream_format(self) -> StreamFormat:
+        return self._recorder.stream_format
+
+    @stream_format.setter
+    def stream_format(self, value: StreamFormat) -> None:
+        self._recorder.stream_format = value
+
+    @property
+    def recording_mode(self) -> RecordingMode:
+        return self._recorder.recording_mode
+
+    @recording_mode.setter
+    def recording_mode(self, value: RecordingMode) -> None:
+        self._recorder.recording_mode = value
+
     @property
     def quality_number(self) -> QualityNumber:
         return self._recorder.quality_number
@@ -280,7 +295,19 @@ class RecordTask:
         self._recorder.quality_number = value
 
     @property
-    def real_quality_number(self) -> QualityNumber:
+    def fmp4_stream_timeout(self) -> int:
+        return self._recorder.fmp4_stream_timeout
+
+    @fmp4_stream_timeout.setter
+    def fmp4_stream_timeout(self, value: int) -> None:
+        self._recorder.fmp4_stream_timeout = value
+
+    @property
+    def real_stream_format(self) -> Optional[StreamFormat]:
+        return self._recorder.real_stream_format
+
+    @property
+    def real_quality_number(self) -> Optional[QualityNumber]:
         return self._recorder.real_quality_number
 
     @property
@@ -339,6 +366,18 @@ class RecordTask:
     def duration_limit(self, value: int) -> None:
         self._recorder.duration_limit = value
 
+    @property
+    def recording_path(self) -> Optional[str]:
+        return self._recorder.recording_path
+
+    @property
+    def metadata(self) -> Optional[MetaData]:
+        return self._recorder.metadata
+
+    @property
+    def stream_profile(self) -> StreamProfile:
+        return self._recorder.stream_profile
+
     @property
     def remux_to_mp4(self) -> bool:
         return self._postprocessor.remux_to_mp4
@@ -421,8 +460,8 @@ class RecordTask:
             await self._recorder.stop()
             await self._postprocessor.stop()
 
-    async def update_info(self) -> None:
-        await self._live.update_info()
+    async def update_info(self, raise_exception: bool = False) -> bool:
+        return await self._live.update_info(raise_exception=raise_exception)
 
     @aio_task_with_room_id
     async def update_session(self) -> None:
@@ -432,7 +471,8 @@ class RecordTask:
         await self._live.deinit()
         await self._live.init()
         self._danmaku_client.session = self._live.session
-        self._danmaku_client.api = self._live.api
+        self._danmaku_client.appapi = self._live.appapi
+        self._danmaku_client.webapi = self._live.webapi
 
         if self._monitor_enabled:
             await self._danmaku_client.start()
@@ -442,12 +482,13 @@ class RecordTask:
         self._setup_live_monitor()
         self._setup_live_event_submitter()
         self._setup_recorder()
+        self._setup_recorder_event_submitter()
         self._setup_postprocessor()
         self._setup_postprocessor_event_submitter()
 
     def _setup_danmaku_client(self) -> None:
         self._danmaku_client = DanmakuClient(
-            self._live.session, self._live.api, self._live.room_id
+            self._live.session, self._live.appapi, self._live.webapi, self._live.room_id
         )
 
     def _setup_live_monitor(self) -> None:
@@ -463,22 +504,10 @@ class RecordTask:
             self._live_monitor,
             self._out_dir,
             self._path_template,
-            buffer_size=self._buffer_size,
-            read_timeout=self._read_timeout,
-            disconnection_timeout=self._disconnection_timeout,
-            danmu_uname=self._danmu_uname,
-            record_gift_send=self._record_gift_send,
-            record_guard_buy=self._record_guard_buy,
-            record_super_chat=self._record_super_chat,
-            save_cover=self._save_cover,
-            save_raw_danmaku=self._save_raw_danmaku,
-            filesize_limit=self._filesize_limit,
-            duration_limit=self._duration_limit,
         )
 
-    def _setup_postprocessor_event_submitter(self) -> None:
-        self._postprocessor_event_submitter = \
-            PostprocessorEventSubmitter(self._postprocessor)
+    def _setup_recorder_event_submitter(self) -> None:
+        self._recorder_event_submitter = RecorderEventSubmitter(self._recorder)
 
     def _setup_postprocessor(self) -> None:
         self._postprocessor = Postprocessor(
@@ -489,8 +518,15 @@ class RecordTask:
             delete_source=self._delete_source,
         )
 
+    def _setup_postprocessor_event_submitter(self) -> None:
+        self._postprocessor_event_submitter = PostprocessorEventSubmitter(
+            self._postprocessor
+        )
+
     async def _destroy(self) -> None:
         self._destroy_postprocessor_event_submitter()
+        self._destroy_postprocessor()
+        self._destroy_recorder_event_submitter()
         self._destroy_recorder()
         self._destroy_live_event_submitter()
         self._destroy_live_monitor()
@@ -508,8 +544,11 @@ class RecordTask:
     def _destroy_recorder(self) -> None:
         del self._recorder
 
-    def _destroy_postprocessor_event_submitter(self) -> None:
-        del self._postprocessor_event_submitter
+    def _destroy_recorder_event_submitter(self) -> None:
+        del self._recorder_event_submitter
 
     def _destroy_postprocessor(self) -> None:
         del self._postprocessor
+
+    def _destroy_postprocessor_event_submitter(self) -> None:
+        del self._postprocessor_event_submitter
