@@ -1,16 +1,16 @@
 import os
 import logging
-import secrets
 from typing import Optional, Tuple
 
-from fastapi import FastAPI, status, Request, Depends, Header
+from fastapi import FastAPI, status, Request, Depends
 from fastapi.responses import JSONResponse
-from fastapi.exceptions import HTTPException
 from fastapi.middleware.cors import CORSMiddleware
 from fastapi.staticfiles import StaticFiles
+from starlette.responses import Response
 from pydantic import ValidationError
 from pkg_resources import resource_filename
 
+from . import security
 from .routers import (
     tasks, settings, application, validation, websockets, update
 )
@@ -18,33 +18,28 @@ from .schemas import ResponseMessage
 from ..setting import EnvSettings, Settings
 from ..application import Application
 from ..exception import NotFoundError, ExistsError, ForbiddenError
+from ..path.helpers import file_exists, create_file
 
 
 logger = logging.getLogger(__name__)
 
 
 _env_settings = EnvSettings()
+_path = os.path.abspath(os.path.expanduser(_env_settings.settings_file))
+if not file_exists(_path):
+    create_file(_path)
+_env_settings.settings_file = _path
+
 _settings = Settings.load(_env_settings.settings_file)
 _settings.update_from_env_settings(_env_settings)
+
 app = Application(_settings)
 
 if _env_settings.api_key is None:
     _dependencies = None
 else:
-    async def validate_api_key(
-        x_api_key: Optional[str] = Header(None)
-    ) -> None:
-        assert _env_settings.api_key is not None
-        if (
-            x_api_key is None or
-            not secrets.compare_digest(x_api_key, _env_settings.api_key)
-        ):
-            raise HTTPException(
-                status_code=status.HTTP_401_UNAUTHORIZED,
-                detail='API key is missing or invalid',
-            )
-
-    _dependencies = [Depends(validate_api_key)]
+    security.api_key = _env_settings.api_key
+    _dependencies = [Depends(security.authenticate)]
 
 api = FastAPI(
     title='Bilibili live streaming recorder web API',
@@ -149,6 +144,20 @@ class WebAppFiles(StaticFiles):
             path = 'index.html'
         return await super().lookup_path(path)
 
+    def file_response(self, full_path: str, *args, **kwargs) -> Response:  # type: ignore # noqa
+        # ignore MIME types from Windows registry
+        # workaround for https://github.com/acgnhiki/blrec/issues/12
+        response = super().file_response(full_path, *args, **kwargs)
+        if full_path.endswith('.js'):
+            js_media_type = 'application/javascript'
+            if response.media_type != js_media_type:
+                response.media_type = js_media_type
+                headers = response.headers
+                headers['content-type'] = js_media_type
+                response.raw_headers = headers.raw
+                del response._headers
+        return response
+
 
 directory = resource_filename(__name__, '../data/webapp')
 api.mount('/', WebAppFiles(directory=directory, html=True), name='webapp')
