@@ -51,6 +51,8 @@ AliasKeyOfSettings = Literal[
     'space',
     'emailNotification',
     'serverchanNotification',
+    'pushdeerNotification',
     'pushplusNotification',
+    'telegramNotification',
     'webhooks',
 ]
